/*
*  Name:      date.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   UESQLC base SQL type date
*  Date:      $Date: 2002/09/25 23:13:45 $
*  Revision:  $Revision: 1.2 $
*
*  Copyright (C) 2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _UESQLC_DATE__
#define _UESQLC_DATE__

#include <mpcl/local_date.hh>
#include "base_type.hh"


///  Name-space for UESQL.
namespace uesql
{

  ///  SQL date handler.
  class TDate               :
    public mpcl::TLocalDate ,
    public TBaseType
  {

    public:

      //
      //  C O N S T R U C T O R S
      //

      ///  Builds a new instance.
      TDate (void)          :
        mpcl::TLocalDate () ,
        TBaseType        () {}

      ///  Builds a new instance from another instance.
      TDate (const TDate& rktDATE) :
        mpcl::TLocalDate (rktDATE) ,
        TBaseType        ()
      {
        setNull (rktDATE.isNull());
      }

      TDate (const EDateType keDATE_TYPE) :
        mpcl::TLocalDate (keDATE_TYPE)    ,
        TBaseType        ()
      {
        setNull (false);
      }

      TDate (const int kiYEAR, const int kiMONTH, const int kiDAY) :
        mpcl::TLocalDate (kiYEAR, kiMONTH, kiDAY)                  ,
        TBaseType        ()
      {
        setNull (false);
      }

      TDate (const char* pkcDATE) :
        mpcl::TLocalDate (eEmpty) ,
        TBaseType        ()
      {
        set (pkcDATE);
      }

      TDate (const std::string& rkyDATE) :
        mpcl::TLocalDate (eEmpty)        ,
        TBaseType        ()
      {
        set (rkyDATE);
      }

      /**
      *  Assigns value \a pkcVALUE.  If \a pkcVALUE is null or points to a zero
      *  length string, then the instance is null.
      *  @pre ( !pkcVALUE || !*pkcVALUE ) => isNull()
      *  @param pkcVALUE Source value.
      */
      TDate& set (const char* pkcVALUE);

      /**
      *  Assigns value \a rkyVALUE.  If \a rkyVALUE is empty, then the instance
      *  is null.
      *  @pre ( rkyVALUE.empty() ) => isNull()
      *  @param rkyVALUE Source value.
      */
      TDate& set (const std::string& rkyVALUE)
      {
        return set (rkyVALUE.c_str());
      }


    public:

      //
      //  S E L E C T O R S
      //

      mpcl::TString get (void) const
      {
        mpcl::TString   yResult;

        if ( isNull() )
        {
          yResult = "null";
        }
        else
        {
          yResult = TLocalDate::get ("'%Y-%m-%d'");
        }
        return yResult;
      }

      /**
      *  Returns date in SQL format.
      *  @param pkcDATE_FORMAT Date format in strftime-like format. It defaults
      *                        to ANSI SQL date  format.  A  valid date  can be
      *                        '1970-12-31'.
      *  @return Date in readable format.
      *  @warning pkcDATE_FORMAT must include the enclosing quotes (').
      */
      mpcl::TString get (const char* pkcDATE_FORMAT) const
      {
        mpcl::TString   yResult;

        if ( isNull() )
        {
          yResult = "null";
        }
        else
        {
          yResult = TLocalDate::get (pkcDATE_FORMAT);
        }
        return yResult;
      }

  };  // class TDate

}  // namespace uesql


#endif  // not _UESQLC_DATE__
