/*
*  Name:      TState.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   State data
*  Date:      $Date: 2002/09/28 18:54:48 $
*  Revision:  $Revision: 1.4 $
*
*  Copyright (C) 2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package for UESQL.
package org.uesqlc;

import java.util.Hashtable;


///  Class for holding the SQLSTATE related variables and values.
public class TState
{

  /**
  *  SQL state parameter (values are implementation defined and using it
  *  is deprecated).
  */
  public static int   SQLCODE = 0;

  /**
  *  SQL extended description (target dependent). This an extended message
  *  for including target dependent description of errors.
  */
  public static String   SQLMESSAGE = "";

  ///  SQL state parameter (values in clause 22, "Status codes").
  public static String   SQLSTATE = "00000";

  /**
  *  Map with SQLSTATE values (strings) to integer translations.
  *  So the integer values are suitable for a \b switch statement.
  */
  private static Hashtable   _tValueToIntMap = new Hashtable();

  //
  //  SQLSTATE class and subclass values.
  //    Classes has been represented as a Java class, and subclasses have
  //    been represented as strings.
  //

  ///  Condition: ambiguous cursor name.
  public static class ambiguous_cursor_name
  {
    public static final String   _kyNO_SUBCLASS = "3C000";
    public static final int      _kiNO_SUBCLASS = 0;
  }

  ///  Condition: cardinality violation.
  public static class cardinality_violation
  {
    public static final String   _kyNO_SUBCLASS = "21000";
    public static final int      _kiNO_SUBCLASS = 1000;
  }
  
  ///  Condition: connection exception.
  public static class connection_exception
  {
    public static final String   _kyNO_SUBCLASS                                       = "08000";
    public static final String   _kyCONNECTION_DOES_NOT_EXIST                         = "08003";
    public static final String   _kyCONNECTION_FAILURE                                = "08006";
    public static final String   _kyCONNECTION_NAME_IN_USE                            = "08002";
    public static final String   _kySQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION       = "08001";
    public static final String   _kySQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION = "08004";
    public static final String   _kyTRANSACTION_RESOLUTION_UNKNOWN                    = "08007";
    public static final int      _kiNO_SUBCLASS                                       = 2000;
    public static final int      _kiCONNECTION_DOES_NOT_EXIST                         = 2001;
    public static final int      _kiCONNECTION_FAILURE                                = 2002;
    public static final int      _kiCONNECTION_NAME_IN_USE                            = 2003;
    public static final int      _kiSQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION       = 2004;
    public static final int      _kiSQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION = 2005;
    public static final int      _kiTRANSACTION_RESOLUTION_UNKNOWN                    = 2006;
  }
  
  ///  Condition: data exception.
  public static class data_exception
  {
    public static final String   _kyNO_SUBCLASS                          = "22000";
    public static final String   _kyCHARACTER_NOT_IN_REPERTOIRE          = "22021";
    public static final String   _kyDATETIME_FIELD_OVERFLOW              = "22008";
    public static final String   _kyDIVISION_BY_ZERO                     = "22012";
    public static final String   _kyERROR_IN_ASSIGNMENT                  = "22005";
    public static final String   _kyINDICATOR_OVERFLOW                   = "22022";
    public static final String   _kyINTERVAL_FIELD_OVERFLOW              = "22015";
    public static final String   _kyINVALID_CHARACTER_VALUE_FOR_CAST     = "22018";
    public static final String   _kyINVALID_DATETIME_FORMAT              = "22007";
    public static final String   _kyINVALID_ESCAPE_CHARACTER             = "22019";
    public static final String   _kyINVALID_ESCAPE_SEQUENCE              = "22025";
    public static final String   _kyINVALID_PARAMETER_VALUE              = "22023";
    public static final String   _kyINVALID_TIME_ZONE_DISPLACEMENT_VALUE = "22009";
    public static final String   _kyNULL_VALUE_NO_INDICATOR_PARAMETER    = "22002";
    public static final String   _kyNUMERIC_VALUE_OUT_OF_RANGE           = "22003";
    public static final String   _kySTRING_DATA_LENGTH_MISMATCH          = "22026";
    public static final String   _kySTRING_DATA_RIGHT_TRUNCATION         = "22001";
    public static final String   _kySUBSTRING_ERROR                      = "22011";
    public static final String   _kyTRIM_ERROR                           = "22027";
    public static final String   _kyUNTERMINATED_C_STRING                = "22024";
    public static final int      _kiNO_SUBCLASS                          = 3000;
    public static final int      _kiCHARACTER_NOT_IN_REPERTOIRE          = 3001;
    public static final int      _kiDATETIME_FIELD_OVERFLOW              = 3002;
    public static final int      _kiDIVISION_BY_ZERO                     = 3003;
    public static final int      _kiERROR_IN_ASSIGNMENT                  = 3004;
    public static final int      _kiINDICATOR_OVERFLOW                   = 3005;
    public static final int      _kiINTERVAL_FIELD_OVERFLOW              = 3006;
    public static final int      _kiINVALID_CHARACTER_VALUE_FOR_CAST     = 3007;
    public static final int      _kiINVALID_DATETIME_FORMAT              = 3008;
    public static final int      _kiINVALID_ESCAPE_CHARACTER             = 3009;
    public static final int      _kiINVALID_ESCAPE_SEQUENCE              = 3010;
    public static final int      _kiINVALID_PARAMETER_VALUE              = 3011;
    public static final int      _kiINVALID_TIME_ZONE_DISPLACEMENT_VALUE = 3012;
    public static final int      _kiNULL_VALUE_NO_INDICATOR_PARAMETER    = 3013;
    public static final int      _kiNUMERIC_VALUE_OUT_OF_RANGE           = 3014;
    public static final int      _kiSTRING_DATA_LENGTH_MISMATCH          = 3015;
    public static final int      _kiSTRING_DATA_RIGHT_TRUNCATION         = 3016;
    public static final int      _kiSUBSTRING_ERROR                      = 3017;
    public static final int      _kiTRIM_ERROR                           = 3018;
    public static final int      _kiUNTERMINATED_C_STRING                = 3019;
  }
  
  ///  Condition: dependent privilege descriptors still exist.
  public static class dependent_privilege_descriptors_still_exist
  {
    public static final String   _kyNO_SUBCLASS = "2B000";
    public static final int      _kiNO_SUBCLASS = 4000;
  }
  
  ///  Condition: dynamic SQL error.
  public static class dynamic_sql_error
  {
    public static final String   _kyNO_SUBCLASS                                        = "07000";
    public static final String   _kyCURSOR_SPECIFICATION_CANNOT_BE_EXECUTED            = "07003";
    public static final String   _kyINVALID_DESCRIPTOR_COUNT                           = "07008";
    public static final String   _kyINVALID_DESCRIPTOR_INDEX                           = "07009";
    public static final String   _kyPREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION      = "07005";
    public static final String   _kyRESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION           = "07006";
    public static final String   _kyUSING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC = "07001";
    public static final String   _kyUSING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC            = "07002";
    public static final String   _kyUSING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS       = "07004";
    public static final String   _kyUSING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS            = "07007";
    public static final int      _kiNO_SUBCLASS                                        = 5000;
    public static final int      _kiCURSOR_SPECIFICATION_CANNOT_BE_EXECUTED            = 5001;
    public static final int      _kiINVALID_DESCRIPTOR_COUNT                           = 5002;
    public static final int      _kiINVALID_DESCRIPTOR_INDEX                           = 5003;
    public static final int      _kiPREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION      = 5004;
    public static final int      _kiRESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION           = 5005;
    public static final int      _kiUSING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC = 5006;
    public static final int      _kiUSING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC            = 5007;
    public static final int      _kiUSING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS       = 5008;
    public static final int      _kiUSING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS            = 5009;
  }
  
  ///  Condition: feature not supported.
  public static class feature_not_supported
  {
    public static final String   _kyNO_SUBCLASS                       = "0A000";
    public static final String   _kyMULTIPLE_ENVIRONMENT_TRANSACTIONS = "0A001";
    public static final int      _kiNO_SUBCLASS                       = 6000;
    public static final int      _kiMULTIPLE_ENVIRONMENT_TRANSACTIONS = 6001;
  }
  
  ///  Condition: integrity constraint violation.
  public static class integrity_constraint_violation
  {
    public static final String   _kyNO_SUBCLASS = "23000";
    public static final int      _kiNO_SUBCLASS = 7000;
  }
  
  ///  Condition: invalid authorization specification.
  public static class invalid_authorization_specification
  {
    public static final String   _kyNO_SUBCLASS = "28000";
    public static final int      _kiNO_SUBCLASS = 8000;
  }
  
  ///  Condition: invalid catalog name.
  public static class invalid_catalog_name
  {
    public static final String   _kyNO_SUBCLASS = "3D000";
    public static final int      _kiNO_SUBCLASS = 9000;
  }
  
  ///  Condition: invalid character set name.
  public static class invalid_character_set_name
  {
    public static final String   _kyNO_SUBCLASS = "2C000";
    public static final int      _kiNO_SUBCLASS = 10000;
  }
  
  ///  Condition: invalid condition number.
  public static class invalid_condition_number
  {
    public static final String   _kyNO_SUBCLASS = "35000";
    public static final int      _kiNO_SUBCLASS = 11000;
  }
  
  ///  Condition: invalid connection name.
  public static class invalid_connection_name
  {
    public static final String   _kyNO_SUBCLASS = "2E000";
    public static final int      _kiNO_SUBCLASS = 12000;
  }
  
  ///  Condition: invalid cursor name.
  public static class invalid_cursor_name
  {
    public static final String   _kyNO_SUBCLASS = "34000";
    public static final int      _kiNO_SUBCLASS = 13000;
  }
  
  ///  Condition: invalid cursor state.
  public static class invalid_cursor_state
  {
    public static final String   _kyNO_SUBCLASS = "24000";
    public static final int      _kiNO_SUBCLASS = 14000;
  }
  
  ///  Condition: invalid schema name.
  public static class invalid_schema_name
  {
    public static final String   _kyNO_SUBCLASS = "3F000";
    public static final int      _kiNO_SUBCLASS = 15000;
  }
  
  ///  Condition: invalid SQL descriptor name.
  public static class invalid_sql_descriptor_name
  {
    public static final String   _kyNO_SUBCLASS = "33000";
    public static final int      _kiNO_SUBCLASS = 16000;
  }
  
  ///  Condition: invalid SQL statement name.
  public static class invalid_sql_statement_name
  {
    public static final String   _kyNO_SUBCLASS = "26000";
    public static final int      _kiNO_SUBCLASS = 17000;
  }
  
  ///  Condition: invalid transaction state.
  public static class invalid_transaction_state
  {
    public static final String   _kyNO_SUBCLASS = "25000";
    public static final int      _kiNO_SUBCLASS = 18000;
  }
  
  ///  Condition: invalid transaction termination.
  public static class invalid_transaction_termination
  {
    public static final String   _kyNO_SUBCLASS = "2D000";
    public static final int      _kiNO_SUBCLASS = 19000;
  }
  
  ///  Condition: no data.
  public static class no_data
  {
    public static final String   _kyNO_SUBCLASS = "02000";
    public static final int      _kiNO_SUBCLASS = 20000;
  }
  
  ///  Condition: remote database access.
  public static class remote_database_access
  {
    public static final String   _kyNO_SUBCLASS = "HZ000";
    public static final int      _kiNO_SUBCLASS = 21000;
  }
  
  ///  Condition: successful completion.
  public static class successful_completion
  {
    public static final String   _kyNO_SUBCLASS = "00000";
    public static final int      _kiNO_SUBCLASS = 22000;
  }
  
  ///  Condition: syntax error or access rule violation.
  public static class syntax_error_or_access_rule_violation
  {
    public static final String   _kyNO_SUBCLASS = "42000";
    public static final int      _kiNO_SUBCLASS = 23000;
  }
  
  ///  Condition: syntax error or access rule violation in direct statement.
  public static class syntax_error_or_access_rule_violation_in_direct_statement
  {
    public static final String   _kyNO_SUBCLASS = "2A000";
    public static final int      _kiNO_SUBCLASS = 24000;
  }
  
  ///  Condition: syntax error or access rule violation in dynamic statement.
  public static class syntax_error_or_access_rule_violation_in_dynamic_statement
  {
    public static final String   _kyNO_SUBCLASS = "37000";
    public static final int      _kiNO_SUBCLASS = 25000;
  }
  
  ///  Condition: transaction rollback.
  public static class transaction_rollback
  {
    public static final String   _kyNO_SUBCLASS                    = "40000";
    public static final String   _kyINTEGRITY_CONSTRAINT_VIOLATION = "40002";
    public static final String   _kySERIALIZATION_FAILURE          = "40001";
    public static final String   _kySTATEMENT_COMPLETION_UNKNOWN   = "40003";
    public static final int      _kiNO_SUBCLASS                    = 26000;
    public static final int      _kiINTEGRITY_CONSTRAINT_VIOLATION = 26001;
    public static final int      _kiSERIALIZATION_FAILURE          = 26002;
    public static final int      _kiSTATEMENT_COMPLETION_UNKNOWN   = 26003;
  }
  
  ///  Condition: triggered data change violation.
  public static class triggered_data_change_violation
  {
    public static final String   _kyNO_SUBCLASS = "27000";
    public static final int      _kiNO_SUBCLASS = 27000;
  }
  
  ///  Condition: warning.
  public static class warning
  {
    public static final String   _kyNO_SUBCLASS                                      = "01000";
    public static final String   _kyCURSOR_OPERATION_CONFLICT                        = "01001";
    public static final String   _kyDISCONNECT_ERROR                                 = "01002";
    public static final String   _kyIMPLICIT_ZERO_BIT_PADDING                        = "01008";
    public static final String   _kyINSUFFICIENT_ITEM_DESCRIPTOR_AREAS               = "01005";
    public static final String   _kyNULL_VALUE_ELIMINATED_IN_SET_FUNCTION            = "01003";
    public static final String   _kyPRIVILEGE_NOT_GRANTED                            = "01007";
    public static final String   _kyPRIVILEGE_NOT_REVOKED                            = "01006";
    public static final String   _kyQUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA = "0100A";
    public static final String   _kySEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA = "01009";
    public static final String   _kySTRING_DATA_RIGHT_TRUNCATION                     = "01004";
    public static final int      _kiNO_SUBCLASS                                      = 28000;
    public static final int      _kiCURSOR_OPERATION_CONFLICT                        = 28001;
    public static final int      _kiDISCONNECT_ERROR                                 = 28002;
    public static final int      _kiIMPLICIT_ZERO_BIT_PADDING                        = 28003;
    public static final int      _kiINSUFFICIENT_ITEM_DESCRIPTOR_AREAS               = 28004;
    public static final int      _kiNULL_VALUE_ELIMINATED_IN_SET_FUNCTION            = 28005;
    public static final int      _kiPRIVILEGE_NOT_GRANTED                            = 28006;
    public static final int      _kiPRIVILEGE_NOT_REVOKED                            = 28007;
    public static final int      _kiQUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA = 28008;
    public static final int      _kiSEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA = 28009;
    public static final int      _kiSTRING_DATA_RIGHT_TRUNCATION                     = 28010;
  }
  
  ///  Condition: with check option violation.
  public static class with_check_option_violation
  {
    public static final String   _kyNO_SUBCLASS = "44000";
    public static final int      _kiNO_SUBCLASS = 29000;
  }
  
  ///  Initializes the SQLSTATE to integer map.
  static
  {

    _tValueToIntMap.put (ambiguous_cursor_name._kyNO_SUBCLASS,                                      new Integer (ambiguous_cursor_name._kiNO_SUBCLASS));
    _tValueToIntMap.put (cardinality_violation._kyNO_SUBCLASS,                                      new Integer (cardinality_violation._kiNO_SUBCLASS));
    _tValueToIntMap.put (connection_exception._kyNO_SUBCLASS,                                       new Integer (connection_exception._kiNO_SUBCLASS));
    _tValueToIntMap.put (connection_exception._kyCONNECTION_DOES_NOT_EXIST,                         new Integer (connection_exception._kiCONNECTION_DOES_NOT_EXIST));
    _tValueToIntMap.put (connection_exception._kyCONNECTION_FAILURE,                                new Integer (connection_exception._kiCONNECTION_FAILURE));
    _tValueToIntMap.put (connection_exception._kyCONNECTION_NAME_IN_USE,                            new Integer (connection_exception._kiCONNECTION_NAME_IN_USE));
    _tValueToIntMap.put (connection_exception._kySQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION,       new Integer (connection_exception._kiSQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION));
    _tValueToIntMap.put (connection_exception._kySQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION, new Integer (connection_exception._kiSQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION));
    _tValueToIntMap.put (connection_exception._kyTRANSACTION_RESOLUTION_UNKNOWN,                    new Integer (connection_exception._kiTRANSACTION_RESOLUTION_UNKNOWN));
    _tValueToIntMap.put (data_exception._kyNO_SUBCLASS,                                             new Integer (data_exception._kiNO_SUBCLASS));
    _tValueToIntMap.put (data_exception._kyCHARACTER_NOT_IN_REPERTOIRE,                             new Integer (data_exception._kiCHARACTER_NOT_IN_REPERTOIRE));
    _tValueToIntMap.put (data_exception._kyDATETIME_FIELD_OVERFLOW,                                 new Integer (data_exception._kiDATETIME_FIELD_OVERFLOW));
    _tValueToIntMap.put (data_exception._kyDIVISION_BY_ZERO,                                        new Integer (data_exception._kiDIVISION_BY_ZERO));
    _tValueToIntMap.put (data_exception._kyERROR_IN_ASSIGNMENT,                                     new Integer (data_exception._kiERROR_IN_ASSIGNMENT));
    _tValueToIntMap.put (data_exception._kyINDICATOR_OVERFLOW,                                      new Integer (data_exception._kiINDICATOR_OVERFLOW));
    _tValueToIntMap.put (data_exception._kyINTERVAL_FIELD_OVERFLOW,                                 new Integer (data_exception._kiINTERVAL_FIELD_OVERFLOW));
    _tValueToIntMap.put (data_exception._kyINVALID_CHARACTER_VALUE_FOR_CAST,                        new Integer (data_exception._kiINVALID_CHARACTER_VALUE_FOR_CAST));
    _tValueToIntMap.put (data_exception._kyINVALID_DATETIME_FORMAT,                                 new Integer (data_exception._kiINVALID_DATETIME_FORMAT));
    _tValueToIntMap.put (data_exception._kyINVALID_ESCAPE_CHARACTER,                                new Integer (data_exception._kiINVALID_ESCAPE_CHARACTER));
    _tValueToIntMap.put (data_exception._kyINVALID_ESCAPE_SEQUENCE,                                 new Integer (data_exception._kiINVALID_ESCAPE_SEQUENCE));
    _tValueToIntMap.put (data_exception._kyINVALID_PARAMETER_VALUE,                                 new Integer (data_exception._kiINVALID_PARAMETER_VALUE));
    _tValueToIntMap.put (data_exception._kyINVALID_TIME_ZONE_DISPLACEMENT_VALUE,                    new Integer (data_exception._kiINVALID_TIME_ZONE_DISPLACEMENT_VALUE));
    _tValueToIntMap.put (data_exception._kyNULL_VALUE_NO_INDICATOR_PARAMETER,                       new Integer (data_exception._kiNULL_VALUE_NO_INDICATOR_PARAMETER));
    _tValueToIntMap.put (data_exception._kyNUMERIC_VALUE_OUT_OF_RANGE,                              new Integer (data_exception._kiNUMERIC_VALUE_OUT_OF_RANGE));
    _tValueToIntMap.put (data_exception._kySTRING_DATA_LENGTH_MISMATCH,                             new Integer (data_exception._kiSTRING_DATA_LENGTH_MISMATCH));
    _tValueToIntMap.put (data_exception._kySTRING_DATA_RIGHT_TRUNCATION,                            new Integer (data_exception._kiSTRING_DATA_RIGHT_TRUNCATION));
    _tValueToIntMap.put (data_exception._kySUBSTRING_ERROR,                                         new Integer (data_exception._kiSUBSTRING_ERROR));
    _tValueToIntMap.put (data_exception._kyTRIM_ERROR,                                              new Integer (data_exception._kiTRIM_ERROR));
    _tValueToIntMap.put (data_exception._kyUNTERMINATED_C_STRING,                                   new Integer (data_exception._kiUNTERMINATED_C_STRING));
    _tValueToIntMap.put (dependent_privilege_descriptors_still_exist._kyNO_SUBCLASS,                new Integer (dependent_privilege_descriptors_still_exist._kiNO_SUBCLASS));
    _tValueToIntMap.put (dynamic_sql_error._kyNO_SUBCLASS,                                          new Integer (dynamic_sql_error._kiNO_SUBCLASS));
    _tValueToIntMap.put (dynamic_sql_error._kyCURSOR_SPECIFICATION_CANNOT_BE_EXECUTED,              new Integer (dynamic_sql_error._kiCURSOR_SPECIFICATION_CANNOT_BE_EXECUTED));
    _tValueToIntMap.put (dynamic_sql_error._kyINVALID_DESCRIPTOR_COUNT,                             new Integer (dynamic_sql_error._kiINVALID_DESCRIPTOR_COUNT));
    _tValueToIntMap.put (dynamic_sql_error._kyINVALID_DESCRIPTOR_INDEX,                             new Integer (dynamic_sql_error._kiINVALID_DESCRIPTOR_INDEX));
    _tValueToIntMap.put (dynamic_sql_error._kyPREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION,        new Integer (dynamic_sql_error._kiPREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION));
    _tValueToIntMap.put (dynamic_sql_error._kyRESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION,             new Integer (dynamic_sql_error._kiRESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION));
    _tValueToIntMap.put (dynamic_sql_error._kyUSING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC,   new Integer (dynamic_sql_error._kiUSING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC));
    _tValueToIntMap.put (dynamic_sql_error._kyUSING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC,              new Integer (dynamic_sql_error._kiUSING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC));
    _tValueToIntMap.put (dynamic_sql_error._kyUSING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS,         new Integer (dynamic_sql_error._kiUSING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS));
    _tValueToIntMap.put (dynamic_sql_error._kyUSING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS,              new Integer (dynamic_sql_error._kiUSING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS));
    _tValueToIntMap.put (feature_not_supported._kyNO_SUBCLASS,                                      new Integer (feature_not_supported._kiNO_SUBCLASS));
    _tValueToIntMap.put (feature_not_supported._kyMULTIPLE_ENVIRONMENT_TRANSACTIONS,                new Integer (feature_not_supported._kiMULTIPLE_ENVIRONMENT_TRANSACTIONS));
    _tValueToIntMap.put (integrity_constraint_violation._kyNO_SUBCLASS,                             new Integer (integrity_constraint_violation._kiNO_SUBCLASS));
    _tValueToIntMap.put (invalid_authorization_specification._kyNO_SUBCLASS,                        new Integer (invalid_authorization_specification._kiNO_SUBCLASS));
    _tValueToIntMap.put (invalid_catalog_name._kyNO_SUBCLASS,                                       new Integer (invalid_catalog_name._kiNO_SUBCLASS));
    _tValueToIntMap.put (invalid_character_set_name._kyNO_SUBCLASS,                                 new Integer (invalid_character_set_name._kiNO_SUBCLASS));
    _tValueToIntMap.put (invalid_condition_number._kyNO_SUBCLASS,                                   new Integer (invalid_condition_number._kiNO_SUBCLASS));
    _tValueToIntMap.put (invalid_connection_name._kyNO_SUBCLASS,                                    new Integer (invalid_connection_name._kiNO_SUBCLASS));
    _tValueToIntMap.put (invalid_cursor_name._kyNO_SUBCLASS,                                        new Integer (invalid_cursor_name._kiNO_SUBCLASS));
    _tValueToIntMap.put (invalid_cursor_state._kyNO_SUBCLASS,                                       new Integer (invalid_cursor_state._kiNO_SUBCLASS));
    _tValueToIntMap.put (invalid_schema_name._kyNO_SUBCLASS,                                        new Integer (invalid_schema_name._kiNO_SUBCLASS));
    _tValueToIntMap.put (invalid_sql_descriptor_name._kyNO_SUBCLASS,                                new Integer (invalid_sql_descriptor_name._kiNO_SUBCLASS));
    _tValueToIntMap.put (invalid_sql_statement_name._kyNO_SUBCLASS,                                 new Integer (invalid_sql_statement_name._kiNO_SUBCLASS));
    _tValueToIntMap.put (invalid_transaction_state._kyNO_SUBCLASS,                                  new Integer (invalid_transaction_state._kiNO_SUBCLASS));
    _tValueToIntMap.put (invalid_transaction_termination._kyNO_SUBCLASS,                            new Integer (invalid_transaction_termination._kiNO_SUBCLASS));
    _tValueToIntMap.put (no_data._kyNO_SUBCLASS,                                                    new Integer (no_data._kiNO_SUBCLASS));
    _tValueToIntMap.put (remote_database_access._kyNO_SUBCLASS,                                     new Integer (remote_database_access._kiNO_SUBCLASS));
    _tValueToIntMap.put (successful_completion._kyNO_SUBCLASS,                                      new Integer (successful_completion._kiNO_SUBCLASS));
    _tValueToIntMap.put (syntax_error_or_access_rule_violation._kyNO_SUBCLASS,                      new Integer (syntax_error_or_access_rule_violation._kiNO_SUBCLASS));
    _tValueToIntMap.put (syntax_error_or_access_rule_violation_in_direct_statement._kyNO_SUBCLASS,  new Integer (syntax_error_or_access_rule_violation_in_direct_statement._kiNO_SUBCLASS));
    _tValueToIntMap.put (syntax_error_or_access_rule_violation_in_dynamic_statement._kyNO_SUBCLASS, new Integer (syntax_error_or_access_rule_violation_in_dynamic_statement._kiNO_SUBCLASS));
    _tValueToIntMap.put (transaction_rollback._kyNO_SUBCLASS,                                       new Integer (transaction_rollback._kiNO_SUBCLASS));
    _tValueToIntMap.put (transaction_rollback._kyINTEGRITY_CONSTRAINT_VIOLATION,                    new Integer (transaction_rollback._kiINTEGRITY_CONSTRAINT_VIOLATION));
    _tValueToIntMap.put (transaction_rollback._kySERIALIZATION_FAILURE,                             new Integer (transaction_rollback._kiSERIALIZATION_FAILURE));
    _tValueToIntMap.put (transaction_rollback._kySTATEMENT_COMPLETION_UNKNOWN,                      new Integer (transaction_rollback._kiSTATEMENT_COMPLETION_UNKNOWN));
    _tValueToIntMap.put (triggered_data_change_violation._kyNO_SUBCLASS,                            new Integer (triggered_data_change_violation._kiNO_SUBCLASS));
    _tValueToIntMap.put (warning._kyNO_SUBCLASS,                                                    new Integer (warning._kiNO_SUBCLASS));
    _tValueToIntMap.put (warning._kyCURSOR_OPERATION_CONFLICT,                                      new Integer (warning._kiCURSOR_OPERATION_CONFLICT));
    _tValueToIntMap.put (warning._kyDISCONNECT_ERROR,                                               new Integer (warning._kiDISCONNECT_ERROR));
    _tValueToIntMap.put (warning._kyIMPLICIT_ZERO_BIT_PADDING,                                      new Integer (warning._kiIMPLICIT_ZERO_BIT_PADDING));
    _tValueToIntMap.put (warning._kyINSUFFICIENT_ITEM_DESCRIPTOR_AREAS,                             new Integer (warning._kiINSUFFICIENT_ITEM_DESCRIPTOR_AREAS));
    _tValueToIntMap.put (warning._kyNULL_VALUE_ELIMINATED_IN_SET_FUNCTION,                          new Integer (warning._kiNULL_VALUE_ELIMINATED_IN_SET_FUNCTION));
    _tValueToIntMap.put (warning._kyPRIVILEGE_NOT_GRANTED,                                          new Integer (warning._kiPRIVILEGE_NOT_GRANTED));
    _tValueToIntMap.put (warning._kyPRIVILEGE_NOT_REVOKED,                                          new Integer (warning._kiPRIVILEGE_NOT_REVOKED));
    _tValueToIntMap.put (warning._kyQUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA,               new Integer (warning._kiQUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA));
    _tValueToIntMap.put (warning._kySEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA,               new Integer (warning._kiSEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA));
    _tValueToIntMap.put (warning._kySTRING_DATA_RIGHT_TRUNCATION,                                   new Integer (warning._kiSTRING_DATA_RIGHT_TRUNCATION));
    _tValueToIntMap.put (with_check_option_violation._kyNO_SUBCLASS,                                new Integer (with_check_option_violation._kiNO_SUBCLASS));

  }  // static

  //
  //  S E L E C T O R S
  //

  /**
  *  Returns true if the query is empty.
  *  @return True if query is empty.
  */
  public static boolean _queryIsEmpty()
  {
    return ( _getUniqueValue() == no_data._kiNO_SUBCLASS );
  }

  /**
  *  Returns true if the query has only a single row.
  *  @return True if query has a single row.
  */
  public static boolean _queryHasSingleRow()
  {
    //
    //  Check if query has generated only *one* row.
    //
    return ( ( _getUniqueValue() != cardinality_violation._kiNO_SUBCLASS ) &&
             ( _getUniqueValue() != no_data._kiNO_SUBCLASS )                );
  }

  /**
  *  Finds the integer value for the current SQLSTATE.
  *  @return The integer value for the current SQLSTATE.
  */
  public static int _getUniqueValue()
  {
    return ((Integer) _tValueToIntMap.get (SQLSTATE)).intValue();
  }

  /**
  *  Finds the integer value for the SQLSTATE in \a kySQLSTATE.
  *  @param kySQLSTATE SQLSTATE value.
  *  @return The integer value for the SQLSTATE in \a kySQLSTATE.
  */
  public static int _getUniqueValue (final String kySQLSTATE)
  {
    return ((Integer) _tValueToIntMap.get (kySQLSTATE)).intValue();
  }

  /**
  *  Gets SQLSTATE class.
  *  @return Class description.
  */
  public static final String _getClassValue()
  {
    return _getClassValue (SQLSTATE);
  }

  /**
  *  Gets SQLSTATE class for \a ySQLSTATE.
  *  @return Class description.
  */
  public static final String _getClassValue (final String kySQLSTATE)
  {

    String   yClass;

    switch (_getUniqueValue (kySQLSTATE))
    {
      case ambiguous_cursor_name._kiNO_SUBCLASS:
      {
        yClass = "ambiguous cursor name";
        break;
      }
      case cardinality_violation._kiNO_SUBCLASS:
      {
        yClass = "cardinality violation";
        break;
      }
      case connection_exception._kiNO_SUBCLASS:
      case connection_exception._kiCONNECTION_DOES_NOT_EXIST:
      case connection_exception._kiCONNECTION_FAILURE:
      case connection_exception._kiCONNECTION_NAME_IN_USE:
      case connection_exception._kiSQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION:
      case connection_exception._kiSQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION:
      case connection_exception._kiTRANSACTION_RESOLUTION_UNKNOWN:
      {
        yClass = "connection exception";
        break;
      }
      case data_exception._kiNO_SUBCLASS:
      case data_exception._kiCHARACTER_NOT_IN_REPERTOIRE:
      case data_exception._kiDATETIME_FIELD_OVERFLOW:
      case data_exception._kiDIVISION_BY_ZERO:
      case data_exception._kiERROR_IN_ASSIGNMENT:
      case data_exception._kiINDICATOR_OVERFLOW:
      case data_exception._kiINTERVAL_FIELD_OVERFLOW:
      case data_exception._kiINVALID_CHARACTER_VALUE_FOR_CAST:
      case data_exception._kiINVALID_DATETIME_FORMAT:
      case data_exception._kiINVALID_ESCAPE_CHARACTER:
      case data_exception._kiINVALID_ESCAPE_SEQUENCE:
      case data_exception._kiINVALID_PARAMETER_VALUE:
      case data_exception._kiINVALID_TIME_ZONE_DISPLACEMENT_VALUE:
      case data_exception._kiNULL_VALUE_NO_INDICATOR_PARAMETER:
      case data_exception._kiNUMERIC_VALUE_OUT_OF_RANGE:
      case data_exception._kiSTRING_DATA_LENGTH_MISMATCH:
      case data_exception._kiSTRING_DATA_RIGHT_TRUNCATION:
      case data_exception._kiSUBSTRING_ERROR:
      case data_exception._kiTRIM_ERROR:
      case data_exception._kiUNTERMINATED_C_STRING:
      {
        yClass = "data exception";
        break;
      }
      case dependent_privilege_descriptors_still_exist._kiNO_SUBCLASS:
      {
        yClass = "dependent privilege descriptors still exist";
        break;
      }
      case dynamic_sql_error._kiNO_SUBCLASS:
      case dynamic_sql_error._kiCURSOR_SPECIFICATION_CANNOT_BE_EXECUTED:
      case dynamic_sql_error._kiINVALID_DESCRIPTOR_COUNT:
      case dynamic_sql_error._kiINVALID_DESCRIPTOR_INDEX:
      case dynamic_sql_error._kiPREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION:
      case dynamic_sql_error._kiRESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION:
      case dynamic_sql_error._kiUSING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC:
      case dynamic_sql_error._kiUSING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC:
      case dynamic_sql_error._kiUSING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS:
      case dynamic_sql_error._kiUSING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS:
      {
        yClass = "dynamic SQL error";
        break;
      }
      case feature_not_supported._kiNO_SUBCLASS:
      case feature_not_supported._kiMULTIPLE_ENVIRONMENT_TRANSACTIONS:
      {
        yClass = "feature not supported";
        break;
      }
      case integrity_constraint_violation._kiNO_SUBCLASS:
      {
        yClass = "integrity constraint violation";
        break;
      }
      case invalid_authorization_specification._kiNO_SUBCLASS:
      {
        yClass = "invalid authorization specification";
        break;
      }
      case invalid_catalog_name._kiNO_SUBCLASS:
      {
        yClass = "invalid catalog name";
        break;
      }
      case invalid_character_set_name._kiNO_SUBCLASS:
      {
        yClass = "invalid character set name";
        break;
      }
      case invalid_condition_number._kiNO_SUBCLASS:
      {
        yClass = "invalid condition number";
        break;
      }
      case invalid_connection_name._kiNO_SUBCLASS:
      {
        yClass = "invalid connection name";
        break;
      }
      case invalid_cursor_name._kiNO_SUBCLASS:
      {
        yClass = "invalid cursor name";
        break;
      }
      case invalid_cursor_state._kiNO_SUBCLASS:
      {
        yClass = "invalid cursor state";
        break;
      }
      case invalid_schema_name._kiNO_SUBCLASS:
      {
        yClass = "invalid schema name";
        break;
      }
      case invalid_sql_descriptor_name._kiNO_SUBCLASS:
      {
        yClass = "invalid SQL descriptor name";
        break;
      }
      case invalid_sql_statement_name._kiNO_SUBCLASS:
      {
        yClass = "invalid SQL statement name";
        break;
      }
      case invalid_transaction_state._kiNO_SUBCLASS:
      {
        yClass = "invalid transaction state";
        break;
      }
      case invalid_transaction_termination._kiNO_SUBCLASS:
      {
        yClass = "invalid transaction termination";
        break;
      }
      case no_data._kiNO_SUBCLASS:
      {
        yClass = "no data";
        break;
      }
      case remote_database_access._kiNO_SUBCLASS:
      {
        yClass = "remote database access";
        break;
      }
      case successful_completion._kiNO_SUBCLASS:
      {
        yClass = "successful completion";
        break;
      }
      case syntax_error_or_access_rule_violation._kiNO_SUBCLASS:
      {
        yClass = "syntax error or access rule violation";
        break;
      }
      case syntax_error_or_access_rule_violation_in_direct_statement._kiNO_SUBCLASS:
      {
        yClass = "syntax error or access rule violation in direct statement";
        break;
      }
      case syntax_error_or_access_rule_violation_in_dynamic_statement._kiNO_SUBCLASS:
      {
        yClass = "syntax error or access rule violation in dynamic statement";
        break;
      }
      case transaction_rollback._kiNO_SUBCLASS:
      case transaction_rollback._kiINTEGRITY_CONSTRAINT_VIOLATION:
      case transaction_rollback._kiSERIALIZATION_FAILURE:
      case transaction_rollback._kiSTATEMENT_COMPLETION_UNKNOWN:
      {
        yClass = "transaction rollback";
        break;
      }
      case triggered_data_change_violation._kiNO_SUBCLASS:
      {
        yClass = "triggered data change violation";
        break;
      }
      case warning._kiNO_SUBCLASS:
      case warning._kiCURSOR_OPERATION_CONFLICT:
      case warning._kiDISCONNECT_ERROR:
      case warning._kiIMPLICIT_ZERO_BIT_PADDING:
      case warning._kiINSUFFICIENT_ITEM_DESCRIPTOR_AREAS:
      case warning._kiNULL_VALUE_ELIMINATED_IN_SET_FUNCTION:
      case warning._kiPRIVILEGE_NOT_GRANTED:
      case warning._kiPRIVILEGE_NOT_REVOKED:
      case warning._kiQUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA:
      case warning._kiSEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA:
      case warning._kiSTRING_DATA_RIGHT_TRUNCATION:
      {
        yClass = "warning";
        break;
      }
      case with_check_option_violation._kiNO_SUBCLASS:
      {
        yClass = "with check option violation";
        break;
      }
      default:
      {
        yClass = "unknown code";
      }
    }
    return yClass;
  
  }  // _getClassValue()
      
  /**
  *  Gets  the  complete  description for the SQLSTATE as  in  the  source
  *  parameter \a kySQLSTATE.
  *  @return Complete description of the SQLSTATE in \a kySQLSTATE.
  */
  public static String _getDescription (final String kySQLSTATE)
  {

    StringBuffer   yDescription = new StringBuffer (_getClassValue (kySQLSTATE));
  
    yDescription.append (": ");
    yDescription.append (_getSubclassValue (kySQLSTATE));
    if ( SQLMESSAGE.length() > 0 )
    {
      yDescription.append (" (");
      yDescription.append (SQLMESSAGE);
      yDescription.append (")");
    }
    return yDescription.toString();

  }  // _getDescription()

  /**
  *  Gets SQLSTATE subclass.
  *  @return Subclass description.
  */
  public static final String _getSubclassValue()
  {
    return _getSubclassValue (SQLSTATE);
  }

  /**
  *  Gets SQLSTATE subclass for \a kySQLSTATE.
  *  @return Subclass description.
  */
  public static final String _getSubclassValue (final String kySQLSTATE)
  {
  
    String   ySubclass = new String();
  
    switch (_getUniqueValue (kySQLSTATE))
    {
      case ambiguous_cursor_name._kiNO_SUBCLASS:
      case cardinality_violation._kiNO_SUBCLASS:
      case connection_exception._kiNO_SUBCLASS:
      case data_exception._kiNO_SUBCLASS:
      case dependent_privilege_descriptors_still_exist._kiNO_SUBCLASS:
      case dynamic_sql_error._kiNO_SUBCLASS:
      case feature_not_supported._kiNO_SUBCLASS:
      case integrity_constraint_violation._kiNO_SUBCLASS:
      case invalid_authorization_specification._kiNO_SUBCLASS:
      case invalid_catalog_name._kiNO_SUBCLASS:
      case invalid_character_set_name._kiNO_SUBCLASS:
      case invalid_condition_number._kiNO_SUBCLASS:
      case invalid_connection_name._kiNO_SUBCLASS:
      case invalid_cursor_name._kiNO_SUBCLASS:
      case invalid_cursor_state._kiNO_SUBCLASS:
      case invalid_schema_name._kiNO_SUBCLASS:
      case invalid_sql_descriptor_name._kiNO_SUBCLASS:
      case invalid_sql_statement_name._kiNO_SUBCLASS:
      case invalid_transaction_state._kiNO_SUBCLASS:
      case invalid_transaction_termination._kiNO_SUBCLASS:
      case no_data._kiNO_SUBCLASS:
      case remote_database_access._kiNO_SUBCLASS:
      case successful_completion._kiNO_SUBCLASS:
      case syntax_error_or_access_rule_violation._kiNO_SUBCLASS:
      case syntax_error_or_access_rule_violation_in_direct_statement._kiNO_SUBCLASS:
      case syntax_error_or_access_rule_violation_in_dynamic_statement._kiNO_SUBCLASS:
      case transaction_rollback._kiNO_SUBCLASS:
      case triggered_data_change_violation._kiNO_SUBCLASS:
      case warning._kiNO_SUBCLASS:
      case with_check_option_violation._kiNO_SUBCLASS:
      {
        ySubclass = "(no subclass)";
        break;
      }
      case connection_exception._kiCONNECTION_DOES_NOT_EXIST:
      {
        ySubclass = "connection does not exist";
        break;
      }
      case connection_exception._kiCONNECTION_FAILURE:
      {
        ySubclass = "connection failure";
        break;
      }
      case connection_exception._kiCONNECTION_NAME_IN_USE:
      {
        ySubclass = "connection name in use";
        break;
      }
      case connection_exception._kiSQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION:
      {
        ySubclass = "SQL-client unable to establish SQL-connection";
        break;
      }
      case connection_exception._kiSQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION:
      {
        ySubclass = "SQL-server rejected establishment of SQL-connection";
        break;
      }
      case connection_exception._kiTRANSACTION_RESOLUTION_UNKNOWN:
      {
        ySubclass = "transaction resolution unknown";
        break;
      }
      case data_exception._kiCHARACTER_NOT_IN_REPERTOIRE:
      {
        ySubclass = "character not in repertoire";
        break;
      }
      case data_exception._kiDATETIME_FIELD_OVERFLOW:
      {
        ySubclass = "datetime field overflow";
        break;
      }
      case data_exception._kiDIVISION_BY_ZERO:
      {
        ySubclass = "division by zero";
        break;
      }
      case data_exception._kiERROR_IN_ASSIGNMENT:
      {
        ySubclass = "error in assignment";
        break;
      }
      case data_exception._kiINDICATOR_OVERFLOW:
      {
        ySubclass = "indicator overflow";
        break;
      }
      case data_exception._kiINTERVAL_FIELD_OVERFLOW:
      {
        ySubclass = "interval field overflow";
        break;
      }
      case data_exception._kiINVALID_CHARACTER_VALUE_FOR_CAST:
      {
        ySubclass = "invalid character value for cast";
        break;
      }
      case data_exception._kiINVALID_DATETIME_FORMAT:
      {
        ySubclass = "invalid datetime format";
        break;
      }
      case data_exception._kiINVALID_ESCAPE_CHARACTER:
      {
        ySubclass = "invalid escape character";
        break;
      }
      case data_exception._kiINVALID_ESCAPE_SEQUENCE:
      {
        ySubclass = "invalid escape sequence";
        break;
      }
      case data_exception._kiINVALID_PARAMETER_VALUE:
      {
        ySubclass = "invalid parameter value";
        break;
      }
      case data_exception._kiINVALID_TIME_ZONE_DISPLACEMENT_VALUE:
      {
        ySubclass = "invalid time zone displacement value";
        break;
      }
      case data_exception._kiNULL_VALUE_NO_INDICATOR_PARAMETER:
      {
        ySubclass = "null value, no indicator parameter";
        break;
      }
      case data_exception._kiNUMERIC_VALUE_OUT_OF_RANGE:
      {
        ySubclass = "numeric value out of range";
        break;
      }
      case data_exception._kiSTRING_DATA_LENGTH_MISMATCH:
      {
        ySubclass = "string data, length mismatch";
        break;
      }
      case data_exception._kiSTRING_DATA_RIGHT_TRUNCATION:
      {
        ySubclass = "string data, right truncation";
        break;
      }
      case data_exception._kiSUBSTRING_ERROR:
      {
        ySubclass = "substring error";
        break;
      }
      case data_exception._kiTRIM_ERROR:
      {
        ySubclass = "trim error";
        break;
      }
      case data_exception._kiUNTERMINATED_C_STRING:
      {
        ySubclass = "unterminated C string";
        break;
      }
      case dynamic_sql_error._kiCURSOR_SPECIFICATION_CANNOT_BE_EXECUTED:
      {
        ySubclass = "cursor specification cannot be executed";
        break;
      }
      case dynamic_sql_error._kiINVALID_DESCRIPTOR_COUNT:
      {
        ySubclass = "invalid descriptor count";
        break;
      }
      case dynamic_sql_error._kiINVALID_DESCRIPTOR_INDEX:
      {
        ySubclass = "invalid descriptor index";
        break;
      }
      case dynamic_sql_error._kiPREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION:
      {
        ySubclass = "prepared statement not a cursor specification";
        break;
      }
      case dynamic_sql_error._kiRESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION:
      {
        ySubclass = "restricted data type attribute violation";
        break;
      }
      case dynamic_sql_error._kiUSING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC:
      {
        ySubclass = "using clause does not match dynamic parameter specifications";
        break;
      }
      case dynamic_sql_error._kiUSING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC:
      {
        ySubclass = "using clause does not match target specifications";
        break;
      }
      case dynamic_sql_error._kiUSING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS:
      {
        ySubclass = "using clause required for dynamic parameters";
        break;
      }
      case dynamic_sql_error._kiUSING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS:
      {
        ySubclass = "using clause required for result fields";
        break;
      }
      case feature_not_supported._kiMULTIPLE_ENVIRONMENT_TRANSACTIONS:
      {
        ySubclass = "multiple environment transactions";
        break;
      }
      case transaction_rollback._kiINTEGRITY_CONSTRAINT_VIOLATION:
      {
        ySubclass = "integrity constraint violation";
        break;
      }
      case transaction_rollback._kiSERIALIZATION_FAILURE:
      {
        ySubclass = "serialization failure";
        break;
      }
      case transaction_rollback._kiSTATEMENT_COMPLETION_UNKNOWN:
      {
        ySubclass = "statement completion unknown";
        break;
      }
      case warning._kiCURSOR_OPERATION_CONFLICT:
      {
        ySubclass = "cursor operation conflict";
        break;
      }
      case warning._kiDISCONNECT_ERROR:
      {
        ySubclass = "disconnect error";
        break;
      }
      case warning._kiIMPLICIT_ZERO_BIT_PADDING:
      {
        ySubclass = "implicit zero-bit padding";
        break;
      }
      case warning._kiINSUFFICIENT_ITEM_DESCRIPTOR_AREAS:
      {
        ySubclass = "insufficient item descriptor areas";
        break;
      }
      case warning._kiNULL_VALUE_ELIMINATED_IN_SET_FUNCTION:
      {
        ySubclass = "null value eliminated in set function";
        break;
      }
      case warning._kiPRIVILEGE_NOT_GRANTED:
      {
        ySubclass = "privilege not granted";
        break;
      }
      case warning._kiPRIVILEGE_NOT_REVOKED:
      {
        ySubclass = "privilege not revoked";
        break;
      }
      case warning._kiQUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA:
      {
        ySubclass = "query expression too long for information schema";
        break;
      }
      case warning._kiSEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA:
      {
        ySubclass = "search condition too long for information schema";
        break;
      }
      case warning._kiSTRING_DATA_RIGHT_TRUNCATION:
      {
        ySubclass = "string data, right truncation";
        break;
      }
      default:
      {
        ySubclass = "unknown code";
      }
    }
    return ySubclass;
  
  }  // _getSubclassValue()

}  // class TState
