#include <mpcl/log.hh>
#include "odbc-3.x.hh"

#line 1 "test-5.uesql.cc"
/*
*  Name:      test-5.uesql.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Test for UESQL
*  Date:      $Date: 2002/09/29 21:23:06 $
*  Revision:  $Revision: 1.7 $
*
*  Copyright (C) 2000-2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/config_processor.hh>
#include <mpcl/log.hh>
#include <mpcl/operating_system.hh>
#include <mpcl/string.hh>
#include <mpcl/test.h>


//
//  L O C A L   V A R I A B L E S
//

static uesql::TSqlstateDescriptor   _tSqlstateDescriptor;
  

//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::TString _GetErrorMessage (void)
{

  using mpcl::TString;

  return ( TString (_tSqlstateDescriptor.getClassValue())    +
           ": "                                              +
           TString (_tSqlstateDescriptor.getSubclassValue()) +
           " (" + uesql::SQLMESSAGE + ")"                    );

}  // _GetErrorMessage()


/**
*  Entry  point.  It uses the  user login 'test/test'  and the schema  location
*  'localhost/test'.
*
*  Parameters:
*
*    {-o, --dbms-server-name}=DBMS-SERVER-NAME
*/
int main (int argc, const char* argv[])
{

  using mpcl::Format;
  using mpcl::TConfigProcessor;
  using std::cerr;
  using std::endl;
  using uesql::ComposeSchemaLocation;
  using uesql::ComposeUserLogin;
  using uesql::TString;
  
  TEST_INIT ("Test for SCHEMA related statements");
  TEST_START_WATCH;

  TString            yVar0;
  TString            yVar1;
  TString            yVar2;
  TString            yCursorName;
  TString            yDbmsUserLogin;
  TString            yDbmsSchemaLocation;
  TString            yDbmsServerName;
  TConfigProcessor   tConfigProcessor (argc, argv);
  const char*        pcDbmsServerName = getenv ("DBMS_SERVER_NAME");

  try
  {
    //
    //  Checks environment variables.
    //
    if ( !pcDbmsServerName )
    {
      pcDbmsServerName = "localhost";
    }
    
    //
    //  Define string constants names for options.
    //
    tConfigProcessor.addOption ("dbms-server-name", "o", pcDbmsServerName);
    tConfigProcessor.processOptions();
    yDbmsServerName     = tConfigProcessor ["dbms-server-name"].getValue();
    yDbmsUserLogin      = ComposeUserLogin      ("test",          "test");
    yDbmsSchemaLocation = ComposeSchemaLocation (yDbmsServerName, "test");
  }
  catch (const mpcl::TBaseException& rktEXCEPTION)
  {
    cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }

  TEST_WRITE_INFO ("DBMS-SCHEMA-LOCATION = %s", yDbmsSchemaLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN      = %s", yDbmsUserLogin.c_str());
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::tLog.also (cerr);
  }
  if ( !yDbmsServerName.empty() )
  {
#line 129 "test-5.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 133 "test-5.uesql.cc"

#line 138 "test-5.uesql.cc"
{
mpcl::TString              hySqlSchemaLocation;
mpcl::TString              hyDatabaseName;
mpcl::TString              hyConnectionName;
mpcl::TString              hyUserLogin;
mpcl::TString::size_type   zSolidusPos;

uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnInputQueue (yDbmsServerName);
uesql::htOdbc3xManager.pushOnInputQueue (yDbmsUserLogin);

//
//  '$z' has been omitted.
//
hySqlSchemaLocation = uesql::htOdbc3xManager.instantiate ("$z");
hyConnectionName    = "DEFAULT";
hyUserLogin         = uesql::htOdbc3xManager.instantiate ("$z");
hySqlSchemaLocation.suppress ('\'');
zSolidusPos = hySqlSchemaLocation.find ('/');
if ( zSolidusPos == mpcl::TString::npos )
{
  hyDatabaseName = hySqlSchemaLocation;
}
else
{
  hyDatabaseName = hySqlSchemaLocation.substr (0, zSolidusPos);
}
if ( uesql::htOdbc3xManager.isConnected (hyConnectionName) )
{
  uesql::htOdbc3xManager.unbindConnection (hyConnectionName);
}
if ( uesql::htOdbc3xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "08002";
  goto error;
}
else
{
  uesql::htOdbc3xManager.connect ( hyConnectionName ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
  {
    goto error;
  }
  else
  {
    uesql::htOdbc3xManager.setCurrentConnection (hyConnectionName);
  }
}
}
#line 154 "test-5.uesql.cc"
{
//
//  Non terminal <schema element list> is not used.
//
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("CREATE SCHEMA TEST_5_SCHEMA  CREATE TABLE TABLE_1 (CODE CHARACTER (3) NOT NULL, WORKER_COUNT INTEGER) CREATE TABLE TABLE_2 (WORK_CODE CHARACTER (3) NOT NULL, NAME CHARACTER (40) NOT NULL, HOURS DECIMAL (5));");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 161 "test-5.uesql.cc"
{
mpcl::TString              hySqlSchemaLocation;
mpcl::TString              hyDatabaseName;
mpcl::TString              hyConnectionName;
mpcl::TString              hyUserLogin;
mpcl::TString::size_type   zSolidusPos;

uesql::htOdbc3xManager.clearQueues();
uesql::htOdbc3xManager.pushOnInputQueue (ComposeSchemaLocation (yDbmsServerName, "test_5_schema"));
uesql::htOdbc3xManager.pushOnInputQueue (yDbmsUserLogin);

hySqlSchemaLocation = uesql::htOdbc3xManager.instantiate ("$z");
hyConnectionName    = uesql::htOdbc3xManager.instantiate ("_'myconnection'");
hyUserLogin         = uesql::htOdbc3xManager.instantiate ("$z");
hySqlSchemaLocation.suppress ('\'');
hyConnectionName.suppress ('\'');
zSolidusPos = hySqlSchemaLocation.find ('/');
if ( zSolidusPos == mpcl::TString::npos )
{
  hyDatabaseName = hySqlSchemaLocation;
}
else
{
  hyDatabaseName = hySqlSchemaLocation.substr (0, zSolidusPos);
}
if ( uesql::htOdbc3xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "08002";
  goto error;
}
else
{
  uesql::htOdbc3xManager.connect ( hyConnectionName ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
  {
    goto error;
  }
  else
  {
    uesql::htOdbc3xManager.setCurrentConnection (hyConnectionName);
  }
}
}
#line 166 "test-5.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("DROP TABLE TABLE_1 CASCADE;");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 167 "test-5.uesql.cc"
{
uesql::htOdbc3xManager.clearQueues();

if ( uesql::htOdbc3xManager.existsDefaultConnection() )
{
  uesql::htOdbc3xManager.resetStatus();
  uesql::htOdbc3xManager.unbindCurrentConnection();
}
else
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection does not exist
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "08003";
  goto error;
}
}
#line 168 "test-5.uesql.cc"
{
mpcl::TString   hyConnectionName;

uesql::htOdbc3xManager.clearQueues();

hyConnectionName = uesql::htOdbc3xManager.instantiate ("DEFAULT");
hyConnectionName.suppress ('\'');
uesql::htOdbc3xManager.setCurrentConnection (hyConnectionName);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
}  // not ODBC 3.x
#line 169 "test-5.uesql.cc"


    //
    //  Sleeps for 5  seconds to  let slow  systems (like POSTGRESQL on a i586)
    //  remove connections properly.
    //
    mpcl::tOperatingSystem.sleep (5);
#line 177 "test-5.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 181 "test-5.uesql.cc"
{
uesql::odbc_3_x::TResultTriple   htResultTriple;

uesql::htOdbc3xManager.clearQueues();

htResultTriple = uesql::htOdbc3xManager.execute ("DROP SCHEMA TEST_5_SCHEMA CASCADE;");
uesql::htOdbc3xManager.diagnose (htResultTriple);
uesql::htOdbc3xManager.clearResult (htResultTriple);
if ( uesql::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
}  // not ODBC 3.x
#line 182 "test-5.uesql.cc"

  }
  goto end;

error:
  cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesql::SQLCODE);

end:
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::tLog.erase (cerr);
  }
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
