#include <mpcl/log.hh>
#include "postgresql-libpq-6.2.x.hh"

#line 1 "test-5.uesql.cc"
/*
*  Name:      test-5.uesql.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Test for UESQL
*  Date:      $Date: 2002/09/29 21:23:06 $
*  Revision:  $Revision: 1.7 $
*
*  Copyright (C) 2000-2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/config_processor.hh>
#include <mpcl/log.hh>
#include <mpcl/operating_system.hh>
#include <mpcl/string.hh>
#include <mpcl/test.h>


//
//  L O C A L   V A R I A B L E S
//

static uesql::TSqlstateDescriptor   _tSqlstateDescriptor;
  

//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::TString _GetErrorMessage (void)
{

  using mpcl::TString;

  return ( TString (_tSqlstateDescriptor.getClassValue())    +
           ": "                                              +
           TString (_tSqlstateDescriptor.getSubclassValue()) +
           " (" + uesql::SQLMESSAGE + ")"                    );

}  // _GetErrorMessage()


/**
*  Entry  point.  It uses the  user login 'test/test'  and the schema  location
*  'localhost/test'.
*
*  Parameters:
*
*    {-o, --dbms-server-name}=DBMS-SERVER-NAME
*/
int main (int argc, const char* argv[])
{

  using mpcl::Format;
  using mpcl::TConfigProcessor;
  using std::cerr;
  using std::endl;
  using uesql::ComposeSchemaLocation;
  using uesql::ComposeUserLogin;
  using uesql::TString;
  
  TEST_INIT ("Test for SCHEMA related statements");
  TEST_START_WATCH;

  TString            yVar0;
  TString            yVar1;
  TString            yVar2;
  TString            yCursorName;
  TString            yDbmsUserLogin;
  TString            yDbmsSchemaLocation;
  TString            yDbmsServerName;
  TConfigProcessor   tConfigProcessor (argc, argv);
  const char*        pcDbmsServerName = getenv ("DBMS_SERVER_NAME");

  try
  {
    //
    //  Checks environment variables.
    //
    if ( !pcDbmsServerName )
    {
      pcDbmsServerName = "localhost";
    }
    
    //
    //  Define string constants names for options.
    //
    tConfigProcessor.addOption ("dbms-server-name", "o", pcDbmsServerName);
    tConfigProcessor.processOptions();
    yDbmsServerName     = tConfigProcessor ["dbms-server-name"].getValue();
    yDbmsUserLogin      = ComposeUserLogin      ("test",          "test");
    yDbmsSchemaLocation = ComposeSchemaLocation (yDbmsServerName, "test");
  }
  catch (const mpcl::TBaseException& rktEXCEPTION)
  {
    cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }

  TEST_WRITE_INFO ("DBMS-SCHEMA-LOCATION = %s", yDbmsSchemaLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN      = %s", yDbmsUserLogin.c_str());
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::tLog.also (cerr);
  }
  if ( !yDbmsServerName.empty() )
  {
#line 129 "test-5.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 133 "test-5.uesql.cc"

#line 138 "test-5.uesql.cc"
{
mpcl::TString   hySqlServerDescriptor;
mpcl::TString   hySqlServerName;
mpcl::TString   hyDatabaseName;
mpcl::TString   hyConnectionName;
mpcl::TString   hyUserLogin;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnInputQueue (yDbmsServerName);
uesql::htPostgresql62xManager.pushOnInputQueue (yDbmsUserLogin);

{
  mpcl::TString::size_type   hzBarIndex;

  hySqlServerDescriptor = uesql::htPostgresql62xManager.instantiate ("$z");
  hyConnectionName      = "DEFAULT";
  hyUserLogin           = uesql::htPostgresql62xManager.instantiate ("$z");
  hySqlServerDescriptor.suppress ('\'');
  hzBarIndex = hySqlServerDescriptor.find ('/');
  if ( hzBarIndex == mpcl::TString::npos )
  {
    hySqlServerName = hySqlServerDescriptor;
  }
  else
  {
    hySqlServerName = hySqlServerDescriptor.substr (0, hzBarIndex);
    hyDatabaseName  = hySqlServerDescriptor.substr (hzBarIndex + 1);
  }
  if ( uesql::htPostgresql62xManager.isConnected (hyConnectionName) )
  {
    uesql::htPostgresql62xManager.unbindConnection (hyConnectionName);
  }
}
if ( uesql::htPostgresql62xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "08002";
  goto error;
}
else
{
  if ( hyDatabaseName.empty() )
  {
    //
    //  If there is no PostgreSQL database then connect to 'template1'
    //  (that should always exists).
    //
    hyDatabaseName = "template1";
  }
  uesql::htPostgresql62xManager.connect ( hyConnectionName ,
                                     hySqlServerName  ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
  {
    goto error;
  }
  else
  {
    uesql::htPostgresql62xManager.setCurrentConnection (hyConnectionName);
  }
}
}
#line 154 "test-5.uesql.cc"
{
PGresult*       ptPgresult;
mpcl::TString   hyConnectionName;
mpcl::TString   hyLastConnectionName = uesql::htPostgresql62xManager.currentConnectionName();

uesql::htPostgresql62xManager.clearQueues();

hyConnectionName = uesql::htPostgresql62xManager.instantiate ("TEST_5_SCHEMA");
hyConnectionName.lowercase();
ptPgresult = uesql::htPostgresql62xManager.execute (mpcl::Format ("CREATE DATABASE %s;", hyConnectionName.c_str()));
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
uesql::htPostgresql62xManager.connect (hyConnectionName, hyConnectionName);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
else
{
  uesql::htPostgresql62xManager.setCurrentConnection (hyConnectionName);
  
{
PGresult*   ptPgresult;

ptPgresult = uesql::htPostgresql62xManager.execute ("CREATE TABLE TABLE_1 (CODE CHAR (3) NOT NULL, WORKER_COUNT INT4);");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
{
PGresult*   ptPgresult;

ptPgresult = uesql::htPostgresql62xManager.execute ("CREATE TABLE TABLE_2 (WORK_CODE CHAR (3) NOT NULL, NAME CHAR (40) NOT NULL, HOURS DECIMAL (5));");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
  uesql::htPostgresql62xManager.unbindCurrentConnection();
  uesql::htPostgresql62xManager.setCurrentConnection (hyLastConnectionName);
}
}
#line 161 "test-5.uesql.cc"
{
mpcl::TString   hySqlServerDescriptor;
mpcl::TString   hySqlServerName;
mpcl::TString   hyDatabaseName;
mpcl::TString   hyConnectionName;
mpcl::TString   hyUserLogin;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnInputQueue (ComposeSchemaLocation (yDbmsServerName, "test_5_schema"));
uesql::htPostgresql62xManager.pushOnInputQueue (yDbmsUserLogin);

{
  mpcl::TString::size_type   hzBarIndex;

  hySqlServerDescriptor = uesql::htPostgresql62xManager.instantiate ("$z");
  hyConnectionName      = uesql::htPostgresql62xManager.instantiate ("_'myconnection'");
  hyUserLogin           = uesql::htPostgresql62xManager.instantiate ("$z");
  hySqlServerDescriptor.suppress ('\'');
  hyConnectionName.suppress ('\'');
  hzBarIndex = hySqlServerDescriptor.find ('/');
  if ( hzBarIndex == mpcl::TString::npos )
  {
    hySqlServerName = hySqlServerDescriptor;
  }
  else
  {
    hySqlServerName = hySqlServerDescriptor.substr (0, hzBarIndex);
    hyDatabaseName  = hySqlServerDescriptor.substr (hzBarIndex + 1);
  }
}
if ( uesql::htPostgresql62xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "08002";
  goto error;
}
else
{
  if ( hyDatabaseName.empty() )
  {
    //
    //  If there is no PostgreSQL database then connect to 'template1'
    //  (that should always exists).
    //
    hyDatabaseName = "template1";
  }
  uesql::htPostgresql62xManager.connect ( hyConnectionName ,
                                     hySqlServerName  ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
  {
    goto error;
  }
  else
  {
    uesql::htPostgresql62xManager.setCurrentConnection (hyConnectionName);
  }
}
}
#line 166 "test-5.uesql.cc"
{
PGresult*   ptPgresult;

//
//  'CASCADE' has been omitted.
//
uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("DROP TABLE TABLE_1;");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 167 "test-5.uesql.cc"
{
uesql::htPostgresql62xManager.clearQueues();

if ( uesql::htPostgresql62xManager.existsDefaultConnection() )
{
  uesql::htPostgresql62xManager.resetStatus();
  uesql::htPostgresql62xManager.unbindCurrentConnection();
}
else
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection does not exist
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "08003";
  goto error;
}
}
#line 168 "test-5.uesql.cc"
{
mpcl::TString   hyConnectionName;

uesql::htPostgresql62xManager.clearQueues();

hyConnectionName = uesql::htPostgresql62xManager.instantiate ("DEFAULT");
hyConnectionName.suppress ('\'');
uesql::htPostgresql62xManager.setCurrentConnection (hyConnectionName);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 169 "test-5.uesql.cc"


    //
    //  Sleeps for 5  seconds to  let slow  systems (like POSTGRESQL on a i586)
    //  remove connections properly.
    //
    mpcl::tOperatingSystem.sleep (5);
#line 177 "test-5.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 181 "test-5.uesql.cc"
{
PGresult*       ptPgresult;
mpcl::TString   hySchemaName;

//
//  'CASCADE' has been omitted.
//
uesql::htPostgresql62xManager.clearQueues();

hySchemaName = "TEST_5_SCHEMA";
hySchemaName.lowercase();
ptPgresult = uesql::htPostgresql62xManager.execute (mpcl::Format ("DROP DATABASE %s;", hySchemaName.c_str()));
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 182 "test-5.uesql.cc"

  }
  goto end;

error:
  cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesql::SQLCODE);

end:
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::tLog.erase (cerr);
  }
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
